import argparse
import numpy as np
import pandas as pd

import matplotlib.pyplot as plt

from plotting_settings import *

def create_parser():
    # Create the parser
    parser = argparse.ArgumentParser(
        description="Benchmark results plotter for Matching Algorithms"
    )

    # Add the arguments
    parser.add_argument(
        "--BruteAlgoFile",
        "-b",
        metavar="brute",
        type=str,
        help="the path to data file from the brute force algorithm",
    )

    parser.add_argument(
        "--TreeAlgoFile",
        "-t",
        metavar="tree",
        type=str,
        help="the path to data file from the tree based algorithm",
    )

    return parser


def calculate_matches_per_second(data, matches):
    matches_per_second_per_rnd_n_msgs = []
    for avg_time in data:
        matches_per_second = np.floor(matches / avg_time)
        matches_per_second_per_rnd_n_msgs.append(matches_per_second)
    return matches_per_second_per_rnd_n_msgs

def plot_benchmark_data(brute_algo_data, tree_algo_data, matches, title, x_label, save_as, x_ticks=None):
    # Create figure and axis
    fig, ax = plt.subplots()
    plt.yscale("log")

    # Calculate mean and standard deviation
    brute_mean = brute_algo_data.mean()
    tree_mean = tree_algo_data.mean()

    if x_ticks:
        brute_mean.index = pd.Index(x_ticks)
        tree_mean.index = pd.Index(x_ticks)

    ax.errorbar(
        brute_mean.index,
        brute_mean, 
        fmt="-o", 
        label="Brute-force Algorithm", 
        color="red",
        markersize=5,
    )

    ax.errorbar(
        tree_mean.index,
        tree_mean,
        fmt="-x",
        label="Stateful Tree-based Algorithm",
        color="green",
        markersize=5,
    )

    ax.boxplot(
        brute_algo_data,
        positions=brute_mean.index,
        widths=0.45,
        patch_artist=True,
        boxprops=dict(facecolor="lightcoral"),
    )

    ax.boxplot(
        tree_algo_data,
        positions=tree_mean.index,
        widths=0.45,
        patch_artist=True,
        boxprops=dict(facecolor="lightgreen"),
    )

    ax.set_xlabel(x_label)
    ax.set_ylabel("Time (s) - log scale")
    ax.set_title(title)

    brute_matches_per_second = calculate_matches_per_second(brute_mean, matches)
    tree_matches_per_second = calculate_matches_per_second(tree_mean, matches)

    ax2 = ax.twinx()
    ax2.set_yscale("symlog")
    ax2.set_ylabel("Matches per second - log scale")
    ax2.plot(brute_mean.index, brute_matches_per_second, "--", color="red", label="Brute-force Algorithm Matches per second")
    ax2.plot(tree_mean.index, tree_matches_per_second, "--", color="green", label="Stateful Tree-based Algorithm Matches per second")
    ax.legend(loc="lower center") #, bbox_to_anchor=(0.08, 1.0))
    # ax2.legend(loc="lower center")
    print(f"Saving plot as {save_as}")
    # plt.tight_layout()
    fig.savefig(save_as + ".pdf")
    plt.show()
