import pandas as pd
from common import create_parser, plot_benchmark_data

common_argparse = create_parser()

common_argparse.add_argument(
    "--WithNoise",
    "-x",
    metavar="with_noise",
    type=bool,
    default=False,
    help="Is the plot with noise data? Default is False",
)

# Parse the arguments
args = common_argparse.parse_args()

# Load and process data
brute_algo_data = (
    pd.read_csv(args.BruteAlgoFile, index_col=0, header=None, delimiter=";").T / 1000
)  # Convert to seconds
tree_algo_data = (
    pd.read_csv(args.TreeAlgoFile, index_col=0, header=None, delimiter=";").T / 1000
)

(N_ITER, N) = brute_algo_data.shape

# Change the index to be the size of the join patterns
brute_algo_data.columns = pd.Index(list(range(1, N + 1)))
tree_algo_data.columns = pd.Index(list(range(1, N + 1)))

matches = 100

if args.WithNoise:
    plot_benchmark_data(
        brute_algo_data,
        tree_algo_data,
        matches,
        f"Benchmark with upto size {N} join patterns without guards with 100 noise messages",
        "Size of join pattern",
        "./plots/SizeWithoutGuardBenchmark_WithNoise_Figure_1_bottom_V2",
    )
else:
    plot_benchmark_data(
        brute_algo_data,
        tree_algo_data,
        matches,
        f"Benchmark with upto size {N} join patterns without guards",
        "Size of join pattern",
        "./plots/SizeWithoutGuardBenchmark_Figure_1_top_V2",
    )
