import pandas as pd

from common import create_parser, plot_benchmark_data

common_argparser = create_parser()

common_argparser.add_argument(
    "--WithNoise",
    "-x",
    metavar="with_noise",
    type=bool,
    default=False,
    help="Is the data with noise? Default is False",
)

common_argparser.add_argument(
    "--WithNonMatchingPayload",
    "-p",
    metavar="with_non_matching_payload",
    type=bool,
    default=False,
    help="Is the data with the non-matching payloads? Default is False",
)

# Parse the arguments
args = common_argparser.parse_args()

# Load and process data
brute_algo_data = (
    pd.read_csv(args.BruteAlgoFile, index_col=0, header=None, delimiter=";").T / 1000
)  # Convert to seconds
tree_algo_data = (
    pd.read_csv(args.TreeAlgoFile, index_col=0, header=None, delimiter=";").T / 1000
)

(N_ITER, N) = brute_algo_data.shape

# Change the index to be the size of the join patterns
brute_algo_data.columns = pd.Index(list(range(1, N + 1)))
tree_algo_data.columns = pd.Index(list(range(1, N + 1)))

matches = 100

if args.WithNoise:
    plot_benchmark_data(
        brute_algo_data,
        tree_algo_data,
        matches,
        f"Benchmark with upto size {N} join patterns with guards with 100 noise messages",
        "Size of join pattern",
        "./plots/SizeWithGuardWithNoiseBenchmark_Figure_2_middle_V2",
    )
elif args.WithNonMatchingPayload:
    plot_benchmark_data(
        brute_algo_data,
        tree_algo_data,
        matches,
        f"Benchmark with upto size {N} join patterns with guards -- non-matching message payloads",
        "Size of join pattern",
        "./plots/SizeWithGuardWithNonMatchingPayloadBenchmark_Figure_2_bottom_V2",
    )
else:
    plot_benchmark_data(
        brute_algo_data,
        tree_algo_data,
        matches,
        f"Benchmark with upto size {N} join patterns with guards",
        "Size of join pattern",
        "./plots/SizeWithGuardBenchmark_Figure_2_top_V2",
    )
