import pandas as pd
from common import create_parser, plot_benchmark_data

common_argparser = create_parser()

common_argparser.add_argument(
    "--matches",
    "-m",
    metavar="matches",
    type=int,
    help="The number of matches the smart house benchmark performs. Used to calculate matches per second",
)

# Parse the arguments
args = common_argparser.parse_args()

# Define file paths
brute_algo_path = args.BruteAlgoFile
tree_algo_path = args.TreeAlgoFile

x_ticks = list(range(0, 33, 4))

# Load and process data
brute_algo_data = (
    pd.read_csv(brute_algo_path, index_col=0, header=None, delimiter=";").T / 1000
)  # Convert to seconds
tree_algo_data = (
    pd.read_csv(tree_algo_path, index_col=0, header=None, delimiter=";").T / 1000
)

(N_ITER, _) = tree_algo_data.shape

# Calculate mean
brute_mean = brute_algo_data.mean()
tree_mean = tree_algo_data.mean()

brute_mean.index = pd.Index(x_ticks)
tree_mean.index = pd.Index(x_ticks)

N_RANDOM_MSGS = tree_mean.index[-1]

plot_benchmark_data(
    brute_algo_data,
    tree_algo_data,
    args.matches,
    f"Smart House benchmark with upto {N_RANDOM_MSGS} random messages. {N_ITER} repetitions",
    "Number of random messages sent",
    "./plots/SmartHouseBenchmark_Figure_3_V2",
    x_ticks=x_ticks,
)