import matplotlib.pyplot as plt
import seaborn as sns

def set_size(width, fraction=1):
    """Set figure dimensions to avoid scaling in LaTeX.

    Parameters
    ----------
    width: float
            Document textwidth or columnwidth in pts
    fraction: float, optional
            Fraction of the width which you wish the figure to occupy

    Returns
    -------
    fig_dim: tuple
            Dimensions of figure in inches

    References
    ----------
    [1] https://jwalton.info/Embed-Publication-Matplotlib-Latex/
    """
    # Width of figure (in pts)
    fig_width_pt = width * fraction

    # Convert from pt to inches
    inches_per_pt = 1 / 72.27

    # Golden ratio to set aesthetic figure height
    # https://disq.us/p/2940ij3
    golden_ratio = (5**.5 - 1) / 2

    # Figure width in inches
    fig_width_in = fig_width_pt * inches_per_pt
    # Figure height in inches
    fig_height_in = fig_width_in * golden_ratio

    fig_dim = (fig_width_in, fig_height_in)

    return fig_dim


# plt.style.available
plt.style.use('seaborn')

# Set the style of the plots to the seaborn style (pretty)
sns.set_theme(style="whitegrid")
plt.rcParams["figure.figsize"] = set_size(width=365)
plt.rcParams["figure.autolayout"] = True
# set dpi
plt.rcParams["figure.dpi"] = 150

tex_fonts = {
    # Use LaTeX to write all text
#     "text.usetex": True,
    'font.family' : 'serif',
    # Use 10pt font in plots, to match 10pt font in document
    "axes.labelsize": 11,
    "font.size": 11,
    # Make the legend/label fonts a little smaller
    "legend.fontsize": 8,
    "xtick.labelsize": 8,
    "ytick.labelsize": 8,
    "axes.titlesize": 8,
}

plt.rcParams.update(tex_fonts)