import pandas as pd
import seaborn as sns

import matplotlib.pyplot as plt


# Set the style of the plots to the seaborn style (pretty)
sns.set_theme(style="whitegrid")
plt.rcParams["figure.figsize"] = [9, 6]
plt.rcParams["figure.autolayout"] = True

tex_fonts = {
    # Use LaTeX to write all text
    "text.usetex": True,
    'font.family' : 'lmodern',
    # Use 10pt font in plots, to match 10pt font in document
    "axes.labelsize": 18,
    "font.size": 18,
    # Make the legend/label fonts a little smaller
    "legend.fontsize": 18,
    "xtick.labelsize": 18,
    "ytick.labelsize": 18,
    "axes.titlesize": 18,
}

plt.rcParams.update(tex_fonts)

# increase the font size of text in the plot
# plt.rcParams["font.size"] = 24

def plot_comparison_boxplots(data1, data2, label1='Data 1', label2='Data 2', title=''):
    # Combine the data into one DataFrame
    df = pd.DataFrame({'Data 1': data1, 'Data 2': data2})
    print(df.info)
    # Set up the plot
    fig, ax = plt.subplots(figsize=(10, 8))

    sns.boxplot(data=[data1, data2], orient='v', width=0.6, showfliers=True)

    # Set the labels and title
    ax.set_ylabel('Time (ms)')
    ax.set_title(title)

    # Set the x-axis tick labels
    ax.set_xticklabels([label1, label2])
    plt.tight_layout()
    # Save the plot
    plt.savefig(f'{title}Boxplot.pdf')
    # Show the plot
    plt.show()

# santa_claus_br = pd.read_csv("./data/2024_04_14_10_20_48_SantaClaus/SantaClaus_BruteForceAlgorithm.csv", index_col=0 , sep=';')
# santa_claus_st = pd.read_csv("./data/2024_04_14_10_20_48_SantaClaus/SantaClaus_StatefulTreeBasedAlgorithm.csv", index_col=0 , sep=';')

# ping_pong_br = pd.read_csv("./data/2024_04_15_14_07_40_PingPong/PingPong_BruteForceAlgorithm.csv", index_col=0 , sep=';')
# ping_pong_st = pd.read_csv("./data/2024_04_15_14_07_40_PingPong/PingPong_StatefulTreeBasedAlgorithm.csv", index_col=0 , sep=';')


# plot_comparison_boxplots(santa_claus_br.columns, santa_claus_st.columns, 'Brute-force Algorithm', 'Stateful Tree-based Algorithm', 'Santa Claus Benchmark Results')
# plot_comparison_boxplots(ping_pong_br.columns, ping_pong_st.columns, 'Brute-force Algorithm', 'Stateful Tree-based Algorithm', 'PingPong Benchmark Results')